﻿#region Copyright and License Notice
// Copyright (C)2010-2013 - Rob Levine and other contributors
// http://configgen.codeplex.com
// 
// This file is part of ConfigGen.
// 
// ConfigGen is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// ConfigGen is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License and 
// the GNU Lesser General Public License along with ConfigGen.  
// If not, see <http://www.gnu.org/licenses/>
#endregion

using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using ConfigGen.ConsoleApp.Core.CommandLineOptionCore;
using ConfigGen.Core;

namespace ConfigGen.ConsoleApp.Core.CommandLineOptions
{
    /// <summary>
    /// Console argument handler to handle the "dump settings file" command line option. This processor is responsible for setting the
    /// <see cref="Preferences.DumpSettingsFile"/> property on the settings object.
    /// </summary>
    public class DumpSettingsFile : ICommandLineOption<ConsoleAppPreferences>
    {
        private string _dumpsettingsFile;

        #region IConsoleArgHandler<Preferences> Members

        /// <summary>
        /// Gets the short name of the command line argument, without the leading hyphen; for this processor: "dsf".
        /// </summary>
        /// <value>"dsf"</value>
        public string ShortName
        {
            get { return "dsf"; }
        }

        /// <summary>
        /// Gets the long name of the command line argument, without the leading hyphens; for this processor: "dump-settings-file"
        /// </summary>
        /// <value>"dump-settings-file"</value>
        public string LongName
        {
            get { return "dump-settings-file"; }
        }

        /// <summary>
        /// Gets a help string containing the args usage for this processor: "&lt;dump settings file&gt;"
        /// </summary>
        /// <value>"&lt;dump settings file&gt;"</value>
        public string ArgsUsage
        {
            get { return "<dump settings file>"; }
        }

        /// <summary>
        /// Gets help text for this command line option.
        /// </summary>
        /// <value>Help text</value>
        public string HelpText
        {
            get
            {
                var dumpSettingsOnly = new DumpSettingsOnly();
                return
                    string.Format(
                        "specifies the file into which the configgen settings should be written. This setting is ignored unless -{0} or -{1} is also specified",
                        dumpSettingsOnly.ShortName,
                        dumpSettingsOnly.LongName);
            }
        }

        /// <summary>
        /// Applies the option represented by this instance to the supplied settings instance.
        /// </summary>
        /// <param name="settings">The settings object.</param>
        public void ApplyOptionToSettings(ConsoleAppPreferences settings)
        {
            var dumpsettingsFile = new FileInfo(_dumpsettingsFile);
            settings.DumpSettingsFile = dumpsettingsFile.FullName;
        }

        /// <summary>
        /// Processes the argument queue by consuming (dequeuing) any arguments used by this instance. For this type of processor, 
        /// a single argument is consumed from the queue, being the settings file path.
        /// </summary>
        /// <param name="args">Args queue from which to consume args</param>
        /// <exception cref="ArgumentException">Thrown if args queue is empty.</exception>
        public void ConsumeArgumentsFromQueue(Queue<string> args)
        {
            if (args.Count < 1)
            {
                var msg = string.Format(CultureInfo.InvariantCulture,
                                        "Settings File switch [-{0} or --{1}] expects a filename argument",
                                        ShortName,
                                        LongName);
                throw new ArgumentException(msg);
            }

            _dumpsettingsFile = args.Dequeue();
        }

        #endregion
    }
}
