﻿#region Copyright and License Notice
// Copyright (C)2010-2013 - Rob Levine and other contributors
// http://configgen.codeplex.com
// 
// This file is part of ConfigGen.
// 
// ConfigGen is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// ConfigGen is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License and 
// the GNU Lesser General Public License along with ConfigGen.  
// If not, see <http://www.gnu.org/licenses/>
#endregion

using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using ConfigGen.ConsoleApp.Core.CommandLineOptionCore;
using ConfigGen.Core;

namespace ConfigGen.ConsoleApp.Core.CommandLineOptions
{    
    /// <summary>
    /// Console argument handler to handle the "output directory" command line option. This processor is responsible for setting the
    /// <see cref="Preferences.OutputDirectory"/> property on the settings object.
    /// </summary>
    public class OutputDirectory : ICommandLineOption<ConsoleAppPreferences>
    {
        private string _outputDirectoryPath;

        #region ICommandLineOption<Preferences> Members

        /// <summary>
        /// Gets the short name of the command line argument, without the leading hyphen; for this processor: "o".
        /// </summary>
        /// <value>"o"</value>
        public string ShortName
        {
            get { return "o"; }
        }

        /// <summary>
        /// Gets the long name of the command line argument, without the leading hyphens; for this processor: "output-dir"
        /// </summary>
        /// <value>"output-dir"</value>
        public string LongName
        {
            get { return "output-directory"; }
        }

        /// <summary>
        /// Gets a help string containing the args usage for this processor: "&lt;output-directory&gt;"
        /// </summary>
        /// <value>"&lt;output-directory&gt;"</value>
        public string ArgsUsage
        {
            get { return "<output-directory>"; }
        }

        /// <summary>
        /// Gets help text for this command line option.
        /// </summary>
        /// <value>Help text</value>
        public string HelpText
        {
            get
            {
                return
                    "specifies the output directory into which to write the generated config files. If this directory does not exist, it will be created."; 
            }
        }

        /// <summary>
        /// Applies the option represented by this instance to the supplied settings instance.
        /// </summary>
        /// <param name="settings">The settings object.</param>
        public void ApplyOptionToSettings(ConsoleAppPreferences settings)
        {
            var outputDir = new DirectoryInfo(_outputDirectoryPath);
            settings.OutputDirectory = outputDir.FullName;
        }

        /// <summary>
        /// Processes the argument queue by consuming (dequeuing) any arguments used by this instance. For this type of processor, 
        /// a single argument is consumed from the queue, being the output directory.
        /// </summary>
        /// <param name="args">Args queue from which to consume args</param>
        /// <exception cref="ArgumentException">Thrown if args queue is empty.</exception>
        public void ConsumeArgumentsFromQueue(Queue<string> args)
        {
            if (args.Count < 1)
            {
                var msg = string.Format(CultureInfo.InvariantCulture,
                                        "Output Directory switch [-{0} or --{1}] expects a directory argument",
                                        ShortName,
                                        LongName);
                throw new ArgumentException(msg);
            }
            _outputDirectoryPath = args.Dequeue();
        }

        #endregion
    }
}
