﻿#region Copyright and License Notice
// Copyright (C)2010-2012 - Rob Levine and other contributors
// http://configgen.codeplex.com
// 
// This file is part of ConfigGen.
// 
// ConfigGen is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// ConfigGen is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License and 
// the GNU Lesser General Public License along with ConfigGen.  
// If not, see <http://www.gnu.org/licenses/>.
#endregion
using System;
using System.Collections.Generic;
using System.IO;
using ConfigGen.Core.ConfigurationSettings;

namespace ConfigGen.Core.SettingsLoader
{
    /// <summary>
    /// 
    /// </summary>
    public class CsvSettingsLoader : ISettingsLoader
    {
        /// <summary>
        /// Loads and returns the configuration settings
        /// </summary>
        /// <param name="args">Array of arguments for the loader (e.g. filename for excel)</param>
        /// <returns>collection of loaded configuration settings.</returns>
        public MachinesConfigurationCollection LoadSettings(string[] args)
        {
            if (args == null)
            {
                throw new ArgumentNullException("args");
            }
            if (args.Length < 1 || args.Length > 2)
            {
                throw new ArgumentException("One or two string arguments expected", "args");
            }
            var settingsFile = args[0];
            if (settingsFile == null) throw new NullReferenceException("arg[0] settingsFile must not be null");
            if (settingsFile.Length == 0) throw new ArgumentException("The supplied settings file path string cannot be zero length");
            var settingsFileInfo = new FileInfo(settingsFile);
            if (!settingsFileInfo.Exists) throw new FileNotFoundException("The specified settings file path was not found: " + settingsFileInfo.FullName,settingsFileInfo.FullName);
            var machineSettings = new List<MachineConfigurationSettings>();
            using (var sr = new StreamReader(settingsFile))
            {
                var columns = new List<string>();
                var line = sr.ReadLine();
                if (!string.IsNullOrEmpty(line))
                {
                    //Parse the headers
                    columns.AddRange(line.Split(new[] {','}));
                }
                if (columns.Count > 2)
                {
                    while (!sr.EndOfStream)
                    {
                        line = sr.ReadLine();
                        if (!string.IsNullOrEmpty(line))
                        {
                            //Parse the rows
                            var row = line.Split(new[] { ',' });
                            var settings = ProcessRow(row, columns);
                            if (settings != null)
                            {
                                machineSettings.Add(settings);
                            }
                        }
                    }
                }
            }
            return new MachinesConfigurationCollection(machineSettings);
        }

        private MachineConfigurationSettings ProcessRow(string[] rowValues, IList<string> columnList)
        {
            var machineName = rowValues[0];
            var configFilePath = rowValues[1];
            if (string.IsNullOrEmpty(machineName) || string.IsNullOrEmpty(configFilePath))
            {
                return null;
            }
            var settings = new List<ConfigurationSetting>();
            for (var i = 2; i < columnList.Count; i++)
            {
                if (rowValues[i].Length > 0)
                    settings.Add(new ConfigurationSetting(columnList[i], rowValues[i]));
            }
            return new MachineConfigurationSettings(machineName, configFilePath, settings);
        }
    }
}
